@@ -48,6 +48,14 @@ define('WEBSERVICE_AUTHMETHOD_SESSION_TOKEN', 2);
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
 class webservice {
+    /**
+     * Only update token last access once per this many seconds. (This constant controls update of
+     * the external tokens last access field. There is a similar define LASTACCESS_UPDATE_SECS
+     * which controls update of the web site last access fields.)
+     *
+     * @var int
+     */
+    const TOKEN_LASTACCESS_UPDATE_SECS = 60;
 
     /**
      * Authenticate user (used by download/upload file scripts)
@@ -209,11 +217,32 @@ class webservice {
         }
 
         // log token access
-        $DB->set_field('external_tokens', 'lastaccess', time(), array('id' => $token->id));
+        self::update_token_lastaccess($token);
 
         return array('user' => $user, 'token' => $token, 'service' => $service);
     }
 
+    /**
+     * Updates the last access time for a token.
+     *
+     * @param \stdClass $token Token object (must include id, lastaccess fields)
+     * @param int $time Time of access (0 = use current time)
+     * @throws dml_exception If database error
+     */
+    public static function update_token_lastaccess($token, int $time = 0) {
+        global $DB;
+
+        if (!$time) {
+            $time = time();
+        }
+
+        // Only update the field if it is a different time from previous request,
+        // so as not to waste database effort.
+        if ($time >= $token->lastaccess + self::TOKEN_LASTACCESS_UPDATE_SECS) {
+            $DB->set_field('external_tokens', 'lastaccess', $time, array('id' => $token->id));
+        }
+    }
+
     /**
      * Allow user to call a service
      *
@@ -402,6 +431,29 @@ class webservice {
         return $token;
     }
 
+    /**
+     * Return a token of an arbitrary user by tokenid, including details of the associated user and the service name.
+     * If no tokens exist an exception is thrown
+     *
+     * The returned value is a stdClass:
+     * ->id token id
+     * ->token
+     * ->firstname user firstname
+     * ->lastname
+     * ->name service name
+     *
+     * @param int $tokenid token id
+     * @return stdClass
+     */
+    public function get_token_by_id_with_details($tokenid) {
+        global $DB;
+        $sql = "SELECT t.id, t.token, u.id AS userid, u.firstname, u.lastname, s.name, t.creatorid
+                FROM {external_tokens} t, {user} u, {external_services} s
+                WHERE t.id=? AND t.tokentype = ? AND s.id = t.externalserviceid AND t.userid = u.id";
+        $token = $DB->get_record_sql($sql, array($tokenid, EXTERNAL_TOKEN_PERMANENT), MUST_EXIST);
+        return $token;
+    }
+
     /**
      * Return a database token record for a token id
      *
@@ -1086,7 +1138,7 @@ abstract class webservice_server implements webservice_server_interface {
         $user = $DB->get_record('user', array('id'=>$token->userid), '*', MUST_EXIST);
 
         // log token access
-        $DB->set_field('external_tokens', 'lastaccess', time(), array('id'=>$token->id));
+        webservice::update_token_lastaccess($token);
 
         return $user;
 
@@ -1103,18 +1155,20 @@ abstract class webservice_server implements webservice_server_interface {
         // Must be the same XXX key name as the external_settings::set_XXX function.
         // Must be the same XXX ws parameter name as 'moodlewssettingXXX'.
         $externalsettings = array(
-            'raw' => false,
-            'fileurl' => true,
-            'filter' =>  false);
+            'raw' => array('default' => false, 'type' => PARAM_BOOL),
+            'fileurl' => array('default' => true, 'type' => PARAM_BOOL),
+            'filter' => array('default' => false, 'type' => PARAM_BOOL),
+            'lang' => array('default' => '', 'type' => PARAM_LANG),
+        );
 
         // Load the external settings with the web service settings.
         $settings = external_settings::get_instance();
-        foreach ($externalsettings as $name => $default) {
+        foreach ($externalsettings as $name => $settingdata) {
 
             $wsparamname = 'moodlewssetting' . $name;
 
             // Retrieve and remove the setting parameter from the request.
-            $value = optional_param($wsparamname, $default, PARAM_BOOL);
+            $value = optional_param($wsparamname, $settingdata['default'], $settingdata['type']);
             unset($_GET[$wsparamname]);
             unset($_POST[$wsparamname]);
 
@@ -1180,6 +1234,8 @@ abstract class webservice_base_server extends webservice_server {
      * @uses die
      */
     public function run() {
+        global $CFG, $SESSION;
+
         // we will probably need a lot of memory in some functions
         raise_memory_limit(MEMORY_EXTRA);
 
@@ -1213,6 +1269,23 @@ abstract class webservice_base_server extends webservice_server {
         $event->set_legacy_logdata(array(SITEID, 'webservice', $this->functionname, '' , getremoteaddr() , 0, $this->userid));
         $event->trigger();
 
+        // Do additional setup stuff.
+        $settings = external_settings::get_instance();
+        $sessionlang = $settings->get_lang();
+        if (!empty($sessionlang)) {
+            $SESSION->lang = $sessionlang;
+        }
+
+        setup_lang_from_browser();
+
+        if (empty($CFG->lang)) {
+            if (empty($SESSION->lang)) {
+                $CFG->lang = 'en';
+            } else {
+                $CFG->lang = $SESSION->lang;
+            }
+        }
+
         // finally, execute the function - any errors are catched by the default exception handler
         $this->execute();
 
